@@ -1,6 +1,6 @@
 /* Gnonlin
- * Copyright (C) <2001> Wim Taymans <wim.taymans@chello.be>
- *               <2004> Edward Hervey <edward@fluendo.com>
+ * Copyright (C) <2001> Wim Taymans <wim.taymans@gmail.com>
+ *               <2004-2008> Edward Hervey <bilboed@bilboed.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,6 +25,19 @@
 #include <string.h>
 #include "gnl.h"
 
+/**
+ * SECTION:gnlobject
+ * @short_description: Base class for GNonLin elements
+ * 
+ * <refsect2>
+ * <para>
+ * GnlObject encapsulates default behaviour and implements standard
+ * properties provided by all the GNonLin elements.
+ * </para>
+ * </refsect2>
+ *  
+ */
+
 typedef struct _GnlPadPrivate GnlPadPrivate;
 
 struct _GnlPadPrivate
@@ -112,6 +125,10 @@ gnl_object_class_init (GnlObjectClass * klass)
   gnlobject_class->prepare = GST_DEBUG_FUNCPTR (gnl_object_prepare_func);
   gnlobject_class->cleanup = GST_DEBUG_FUNCPTR (gnl_object_cleanup_func);
 
+  /**
+   * GnlObject:start:
+   *
+   */
   g_object_class_install_property (gobject_class, ARG_START,
       g_param_spec_uint64 ("start", "Start",
           "The start position relative to the parent",
@@ -181,7 +198,7 @@ gnl_object_init (GnlObject * object, GnlObjectClass * klass)
 static void
 gnl_object_dispose (GObject * object)
 {
-  GnlObject *gnl = GNL_OBJECT (object);
+  GnlObject *gnl = (GnlObject *) object;
 
   if (gnl->caps) {
     gst_caps_unref (gnl->caps);
@@ -470,8 +487,8 @@ translate_incoming_seek (GnlObject * object, GstEvent * event)
 
   GST_DEBUG_OBJECT (object,
       "SENDING SEEK rate:%f, format:TIME, flags:%d, curtype:%d, stoptype:SET, %"
-		    GST_TIME_FORMAT " -- %" GST_TIME_FORMAT, nrate, flags, ncurtype,
-		    GST_TIME_ARGS (ncur), GST_TIME_ARGS (nstop));
+      GST_TIME_FORMAT " -- %" GST_TIME_FORMAT, nrate, flags, ncurtype,
+      GST_TIME_ARGS (ncur), GST_TIME_ARGS (nstop));
 
   event2 = gst_event_new_seek (nrate, GST_FORMAT_TIME, flags,
       ncurtype, (gint64) ncur, GST_SEEK_TYPE_SET, (gint64) nstop);
@@ -662,10 +679,10 @@ ghostpad_event_function (GstPad * ghostpad, GstEvent * event)
         event = translate_incoming_seek (object, event);
       } else if (GST_EVENT_TYPE (event) == GST_EVENT_QOS) {
 
-	/* FIXME : Implement proper QoS time-shifting, for the time being it's
-	 * just ignored. See #398453 */
-	gst_event_unref (event);
-	goto beach;
+        /* FIXME : Implement proper QoS time-shifting, for the time being it's
+         * just ignored. See #398453 */
+        gst_event_unref (event);
+        goto beach;
       }
       break;
     default:
@@ -677,7 +694,7 @@ ghostpad_event_function (GstPad * ghostpad, GstEvent * event)
   GST_DEBUG_OBJECT (ghostpad, "Returned from calling priv->eventfunc : %d",
       ret);
 
- beach:
+beach:
   return ret;
 
   /* ERRORS */
@@ -725,10 +742,17 @@ static void
 control_internal_pad (GstPad * ghostpad, GnlObject * object)
 {
   GnlPadPrivate *priv;
-  GnlPadPrivate *privghost = gst_pad_get_element_private (ghostpad);
-  GstPad *target = gst_ghost_pad_get_target (GST_GHOST_PAD (ghostpad));
+  GnlPadPrivate *privghost;
+  GstPad *target;
   GstPad *internal;
 
+  if (!ghostpad) {
+    GST_DEBUG_OBJECT (object, "We don't have a valid ghostpad !");
+    return;
+  }
+  privghost = gst_pad_get_element_private (ghostpad);
+  target = gst_ghost_pad_get_target (GST_GHOST_PAD (ghostpad));
+
   if (!target) {
     GST_DEBUG_OBJECT (ghostpad,
         "ghostpad doesn't have a target, no need to control the internal pad");
@@ -826,11 +850,10 @@ gnl_object_ghost_pad (GnlObject * object, const gchar * name, GstPad * target)
   return gnl_object_ghost_pad_full (object, name, target, FALSE);
 }
 
-/**
+/*
  * gnl_object_ghost_pad_no_target:
  * /!\ Doesn't add the pad to the GnlObject....
-*/
-
+ */
 GstPad *
 gnl_object_ghost_pad_no_target (GnlObject * object, const gchar * name,
     GstPadDirection dir)
@@ -973,7 +996,7 @@ beach:
 static void
 gnl_object_handle_message (GstBin * bin, GstMessage * message)
 {
-  GnlObject *object = GNL_OBJECT (bin);
+  GnlObject *object = (GnlObject *) bin;
 
   GST_DEBUG_OBJECT (object, "message:%s",
       gst_message_type_get_name (GST_MESSAGE_TYPE (message)));
@@ -1050,7 +1073,7 @@ static void
 gnl_object_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GnlObject *gnlobject = GNL_OBJECT (object);
+  GnlObject *gnlobject = (GnlObject *) object;
 
   g_return_if_fail (GNL_IS_OBJECT (object));
 
@@ -1089,11 +1112,7 @@ static void
 gnl_object_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
-  GnlObject *gnlobject;
-
-  g_return_if_fail (GNL_IS_OBJECT (object));
-
-  gnlobject = GNL_OBJECT (object);
+  GnlObject *gnlobject = (GnlObject *) object;
 
   switch (prop_id) {
     case ARG_START:
