@@ -1,6 +1,6 @@
 /* GStreamer
- * Copyright (C) 2001 Wim Taymans <wim.taymans@chello.be>
- *               2004 Edward Hervey <bilboed@bilboed.com>
+ * Copyright (C) 2001 Wim Taymans <wim.taymans@gmail.com>
+ *               2004-2008 Edward Hervey <bilboed@bilboed.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,13 +25,26 @@
 
 #include "gnl.h"
 
+/**
+ * SECTION:element-gnloperation
+ * @short_description: Encapsulates filters/effects for use with GnlObjects
+ *
+ * <refsect2>
+ * <para>
+ * A GnlOperation performs a transformation or mixing operation on the
+ * data from one or more #GnlSources, which is used to implement filters or 
+ * effects.
+ * </para>
+ * </refsect2>
+ */
+
 GST_BOILERPLATE (GnlOperation, gnl_operation, GnlObject, GNL_TYPE_OBJECT);
 
 static GstElementDetails gnl_operation_details =
 GST_ELEMENT_DETAILS ("GNonLin Operation",
     "Filter/Editor",
     "Encapsulates filters/effects for use with GNL Objects",
-    "Wim Taymans <wim.taymans@chello.be>, Edward Hervey <bilboed@bilboed.com>");
+    "Wim Taymans <wim.taymans@gmail.com>, Edward Hervey <bilboed@bilboed.com>");
 
 static GstStaticPadTemplate gnl_operation_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -98,6 +111,13 @@ gnl_operation_class_init (GnlOperationClass * klass)
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gnl_operation_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gnl_operation_get_property);
 
+  /**
+   * GnlOperation:sinks:
+   *
+   * Specifies the number of sink pads the operation should provide.
+   * If the sinks property is -1 (the default) pads are only created as
+   * demanded via get_request_pad() calls on the element.
+   */
   g_object_class_install_property (gobject_class, ARG_SINKS,
       g_param_spec_int ("sinks", "Sinks",
           "Number of input sinks (-1 for automatic handling)", -1, G_MAXINT, -1,
@@ -124,7 +144,7 @@ gnl_operation_class_init (GnlOperationClass * klass)
 static void
 gnl_operation_finalize (GObject * object)
 {
-  GnlOperation *oper = GNL_OPERATION (object);
+  GnlOperation *oper = (GnlOperation *) object;
 
   g_list_free (oper->sinks);
 
@@ -198,8 +218,7 @@ element_is_valid_filter (GstElement * element, gboolean * isdynamic)
     if (template->direction == GST_PAD_SRC)
       havesrc = TRUE;
     else if (template->direction == GST_PAD_SINK) {
-      if (isdynamic
-          && (GST_PAD_TEMPLATE_PRESENCE (template) == GST_PAD_REQUEST))
+      if (!havesink && (template->presence == GST_PAD_REQUEST))
         *isdynamic = TRUE;
       havesink = TRUE;
     }
@@ -208,9 +227,9 @@ element_is_valid_filter (GstElement * element, gboolean * isdynamic)
   return (havesink && havesrc);
 }
 
-/**
+/*
  * get_src_pad:
- * #element: a #GstElement
+ * element: a #GstElement
  *
  * Returns: The src pad for the given element. A reference was added to the
  * returned pad, remove it when you don't need that pad anymore.
@@ -234,10 +253,46 @@ get_src_pad (GstElement * element)
   return srcpad;
 }
 
+/* get_nb_static_sinks:
+ * 
+ * Returns : The number of static sink pads of the controlled element.
+ */
+static guint
+get_nb_static_sinks (GnlOperation * oper)
+{
+  GstIterator *sinkpads;
+  gboolean done = FALSE;
+  gpointer val;
+  guint nbsinks = 0;
+
+  sinkpads = gst_element_iterate_sink_pads (oper->element);
+
+  while (!done) {
+    switch (gst_iterator_next (sinkpads, &val)) {
+      case GST_ITERATOR_OK:
+        nbsinks++;
+        break;
+      case GST_ITERATOR_RESYNC:
+        nbsinks = 0;
+        gst_iterator_resync (sinkpads);
+        break;
+      default:
+        /* ERROR and DONE */
+        done = TRUE;
+        break;
+    }
+  }
+  gst_iterator_free (sinkpads);
+
+  GST_DEBUG ("We found %d static sinks", nbsinks);
+
+  return nbsinks;
+}
+
 static gboolean
 gnl_operation_add_element (GstBin * bin, GstElement * element)
 {
-  GnlOperation *operation = GNL_OPERATION (bin);
+  GnlOperation *operation = (GnlOperation *) bin;
   gboolean res = FALSE;
   gboolean isdynamic;
 
@@ -250,7 +305,8 @@ gnl_operation_add_element (GstBin * bin, GstElement * element)
   } else {
     if (!element_is_valid_filter (element, &isdynamic)) {
       GST_WARNING_OBJECT (operation,
-          "Element %s is not a valid filter element");
+          "Element %s is not a valid filter element",
+          GST_ELEMENT_NAME (element));
     } else {
       if ((res = GST_BIN_CLASS (parent_class)->add_element (bin, element))) {
         GstPad *srcpad;
@@ -262,16 +318,20 @@ gnl_operation_add_element (GstBin * bin, GstElement * element)
         operation->element = element;
         operation->dynamicsinks = isdynamic;
 
+        /* Source ghostpad */
         if (!operation->ghostpad) {
           operation->ghostpad =
               gst_ghost_pad_new_no_target ("src", GST_PAD_SRC);
-	  gst_pad_set_active (operation->ghostpad, TRUE);
+          gst_pad_set_active (operation->ghostpad, TRUE);
           gst_element_add_pad ((GstElement *) bin, operation->ghostpad);
         }
         gst_ghost_pad_set_target ((GstGhostPad *) operation->ghostpad, srcpad);
-
         gst_object_unref (srcpad);
 
+        /* Figure out number of static sink pads */
+        operation->num_sinks = get_nb_static_sinks (operation);
+
+        /* Finally sync the ghostpads with the real pads */
         synchronize_sinks (operation);
       }
     }
@@ -283,7 +343,7 @@ gnl_operation_add_element (GstBin * bin, GstElement * element)
 static gboolean
 gnl_operation_remove_element (GstBin * bin, GstElement * element)
 {
-  GnlOperation *operation = GNL_OPERATION (bin);
+  GnlOperation *operation = (GnlOperation *) bin;
   gboolean res = FALSE;
 
   if (operation->element) {
@@ -310,11 +370,7 @@ static void
 gnl_operation_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
 {
-  GnlOperation *operation;
-
-  g_return_if_fail (GNL_IS_OPERATION (object));
-
-  operation = GNL_OPERATION (object);
+  GnlOperation *operation = (GnlOperation *) object;
 
   switch (prop_id) {
     case ARG_SINKS:
@@ -330,11 +386,7 @@ static void
 gnl_operation_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
 {
-  GnlOperation *operation;
-
-  g_return_if_fail (GNL_IS_OPERATION (object));
-
-  operation = GNL_OPERATION (object);
+  GnlOperation *operation = (GnlOperation *) object;
 
   switch (prop_id) {
     case ARG_SINKS:
@@ -380,8 +432,8 @@ get_unused_static_sink_pad (GnlOperation * operation)
             GstGhostPad *gpad = (GstGhostPad *) tmp->data;
             GstPad *target = gst_ghost_pad_get_target (gpad);
 
-	    GST_LOG ("found ghostpad with target %s:%s",
-		     GST_DEBUG_PAD_NAME (target));
+            GST_LOG ("found ghostpad with target %s:%s",
+                GST_DEBUG_PAD_NAME (target));
 
             if (target) {
               if (target == pad)
@@ -390,15 +442,15 @@ get_unused_static_sink_pad (GnlOperation * operation)
             }
           }
 
-	  /* 2. if not taken, return that pad */
+          /* 2. if not taken, return that pad */
           if (!istaken) {
             ret = pad;
             done = TRUE;
           } else {
             gst_object_unref (pad);
-	  }
+          }
         } else
-	  gst_object_unref (pad);
+          gst_object_unref (pad);
         break;
       }
       case GST_ITERATOR_RESYNC:
@@ -424,29 +476,82 @@ get_unused_static_sink_pad (GnlOperation * operation)
   return ret;
 }
 
+static GstPad *
+get_unlinked_sink_ghost_pad (GnlOperation * operation)
+{
+  GstIterator *pads;
+  gboolean done = FALSE;
+  gpointer val;
+  GstPad *ret = NULL;
+
+  if (!operation->element)
+    return NULL;
+
+  pads = gst_element_iterate_sink_pads ((GstElement *) operation);
+
+  while (!done) {
+    switch (gst_iterator_next (pads, &val)) {
+      case GST_ITERATOR_OK:
+      {
+        GstPad *pad = (GstPad *) val;
+        GstPad *peer = gst_pad_get_peer (pad);
+
+        if (peer == NULL) {
+          ret = pad;
+          done = TRUE;
+        } else
+          gst_object_unref ((GstObject *) pad);
+        break;
+      }
+      case GST_ITERATOR_RESYNC:
+        if (ret)
+          gst_object_unref (ret);
+        ret = NULL;
+        gst_iterator_resync (pads);
+        break;
+      default:
+        /* ERROR and DONE */
+        done = TRUE;
+        break;
+    }
+  }
+  gst_iterator_free (pads);
+
+  if (ret)
+    GST_DEBUG_OBJECT (operation, "found unlinked ghost sink pad %s:%s",
+        GST_DEBUG_PAD_NAME (ret));
+  else
+    GST_DEBUG_OBJECT (operation, "Couldn't find an unlinked ghost sink pad");
+
+  return ret;
+
+}
+
 static GstPad *
 get_request_sink_pad (GnlOperation * operation)
 {
-  GstPad * pad = NULL;
-  GList * templates;
-  
+  GstPad *pad = NULL;
+  GList *templates;
+
   if (!operation->element)
     return NULL;
 
   templates = gst_element_class_get_pad_template_list
-    (GST_ELEMENT_GET_CLASS (operation->element));
+      (GST_ELEMENT_GET_CLASS (operation->element));
 
-  for ( ; templates; templates = g_list_next(templates)) {
-    GstPadTemplate * templ = (GstPadTemplate*) templates->data;
+  for (; templates; templates = g_list_next (templates)) {
+    GstPadTemplate *templ = (GstPadTemplate *) templates->data;
 
     GST_LOG_OBJECT (operation->element, "Trying template %s",
-		    GST_PAD_TEMPLATE_NAME_TEMPLATE (templ));
+        GST_PAD_TEMPLATE_NAME_TEMPLATE (templ));
 
-    if ( (GST_PAD_TEMPLATE_DIRECTION(templ) == GST_PAD_SINK) &&
-	 (GST_PAD_TEMPLATE_PRESENCE(templ) == GST_PAD_REQUEST) ) {
-      pad = gst_element_get_request_pad (operation->element, GST_PAD_TEMPLATE_NAME_TEMPLATE(templ));
+    if ((GST_PAD_TEMPLATE_DIRECTION (templ) == GST_PAD_SINK) &&
+        (GST_PAD_TEMPLATE_PRESENCE (templ) == GST_PAD_REQUEST)) {
+      pad =
+          gst_element_get_request_pad (operation->element,
+          GST_PAD_TEMPLATE_NAME_TEMPLATE (templ));
       if (pad)
-	break;
+        break;
     }
   }
 
@@ -457,19 +562,17 @@ static GstPad *
 add_sink_pad (GnlOperation * operation)
 {
   GstPad *gpad = NULL;
+  GstPad *ret = NULL;
 
   if (!operation->element)
     return NULL;
-  
+
   /* FIXME : implement */
   GST_LOG_OBJECT (operation, "element:%s , dynamicsinks:%d",
-		  GST_ELEMENT_NAME (operation->element),
-		  operation->dynamicsinks);
+      GST_ELEMENT_NAME (operation->element), operation->dynamicsinks);
 
 
   if (!operation->dynamicsinks) {
-    GstPad *ret;
-
     /* static sink pads */
     ret = get_unused_static_sink_pad (operation);
     if (ret) {
@@ -477,11 +580,9 @@ add_sink_pad (GnlOperation * operation)
       gst_object_unref (ret);
     }
   }
-  
-  if (!gpad) {
-    GstPad * ret;
-    /* request sink pads */
 
+  if (!gpad) {
+    /* request sink pads */
     ret = get_request_sink_pad (operation);
     if (ret) {
       gpad = gst_ghost_pad_new (GST_PAD_NAME (ret), ret);
@@ -492,17 +593,23 @@ add_sink_pad (GnlOperation * operation)
   if (gpad) {
     gst_pad_set_active (gpad, TRUE);
     gst_element_add_pad ((GstElement *) operation, gpad);
-    operation->sinks = g_list_append(operation->sinks, gpad);
+    operation->sinks = g_list_append (operation->sinks, gpad);
     operation->realsinks++;
+    GST_DEBUG ("Created new pad %s:%s ghosting %s:%s",
+        GST_DEBUG_PAD_NAME (gpad), GST_DEBUG_PAD_NAME (ret));
+  } else {
+    GST_WARNING ("Couldn't find a usable sink pad!");
   }
 
   return gpad;
 }
 
 static gboolean
-remove_sink_pad (GnlOperation * operation)
+remove_sink_pad (GnlOperation * operation, GstPad * sinkpad)
 {
-  /* FIXME : implement */
+  gboolean ret = TRUE;
+
+  GST_DEBUG ("sinkpad %s:%s", GST_DEBUG_PAD_NAME (sinkpad));
 
   /*
      We can't remove any random pad.
@@ -510,7 +617,25 @@ remove_sink_pad (GnlOperation * operation)
      thread-safe way.
    */
 
-  return TRUE;
+  if ((sinkpad == NULL) && operation->dynamicsinks) {
+    /* Find an unlinked sinkpad */
+    if ((sinkpad = get_unlinked_sink_ghost_pad (operation)) == NULL) {
+      ret = FALSE;
+      goto beach;
+    }
+  }
+
+  if (sinkpad) {
+    GstPad *target = gst_ghost_pad_get_target ((GstGhostPad *) sinkpad);
+
+    /* release the target pad */
+    gst_element_release_request_pad (operation->element, target);
+    operation->sinks = g_list_remove (operation->sinks, sinkpad);
+    gst_element_remove_pad ((GstElement *) operation, sinkpad);
+  }
+
+beach:
+  return ret;
 }
 
 static void
@@ -519,30 +644,27 @@ synchronize_sinks (GnlOperation * operation)
 
   GST_DEBUG_OBJECT (operation, "num_sinks:%d , realsinks:%d",
       operation->num_sinks, operation->realsinks);
-  if ((operation->num_sinks == -1) ||
+  if ((operation->dynamicsinks) ||
       (operation->num_sinks == operation->realsinks))
     return;
 
   if (operation->num_sinks > operation->realsinks) {
     while (operation->num_sinks > operation->realsinks) /* Add pad */
       if (!(add_sink_pad (operation))) {
-	break;
+        break;
       }
   } else {
     /* Remove pad */
     /* FIXME, which one do we remove ? :) */
-    remove_sink_pad (operation);
+    remove_sink_pad (operation, NULL);
   }
 }
 
 static gboolean
 gnl_operation_prepare (GnlObject * object)
 {
-  if (!(GNL_OPERATION (object)->element))
-    return FALSE;
-
   /* Prepare the pads */
-  synchronize_sinks (GNL_OPERATION (object));
+  synchronize_sinks ((GnlOperation *) object);
 
   return TRUE;
 }
@@ -554,6 +676,8 @@ gnl_operation_request_new_pad (GstElement * element, GstPadTemplate * templ,
   GnlOperation *operation = (GnlOperation *) element;
   GstPad *ret;
 
+  GST_DEBUG ("template:%s name:%s", templ->name_template, name);
+
   if (operation->num_sinks == operation->realsinks) {
     GST_WARNING_OBJECT (element,
         "We already have the maximum number of pads : %d",
@@ -569,5 +693,7 @@ gnl_operation_request_new_pad (GstElement * element, GstPadTemplate * templ,
 static void
 gnl_operation_release_pad (GstElement * element, GstPad * pad)
 {
+  GST_DEBUG ("pad %s:%s", GST_DEBUG_PAD_NAME (pad));
 
+  remove_sink_pad ((GnlOperation *) element, pad);
 }
